/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL inovat OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCL. NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.inovat.lib.funktionen.datum;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.Activator;
import de.inovat.buv.inovat.lib.debug.Log;

import org.eclipse.swt.widgets.DateTime;

//~ JDK IMPORTE ===============================================================

import java.util.Calendar;
import java.util.GregorianCalendar;

//~ KLASSEN ===================================================================

/**
 * Klasse enthlt statische Methoden fr die Arbeit mit dem Datum als Typ
 * org.eclipse.swt.widgets.DateTime und Typ long (Datum in Millisekunden).
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 12 $ / $Date: 2008-05-21 16:48:36 +0200 (Mi, 21 Mai
 *          2008) $ / ($Author: LG $)
 */
public class DatumFunktionen {

    /** Anzahl der Millisekunden in einem Tag (1000 * 60 * 60 * 24). */
    public static final long ANZAHL_MILLISEKUNDEN_PRO_TAG = 86400000l;

    //~ METHODEN ==============================================================

    /**
     * Erstelle ISOZeitstempel fr das aktuelle Datum:
     * 'Dyyyy-mm-ddThhmiss'.
     *
     * @return ISOZeitstempel.
     */
    public static String erstelleISOZeitStempel() {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(getAktuelleDateTime());

        String yyyy = "" + calender.get(Calendar.YEAR);

        // Monat+1, da Monat fngt von 0 an.
        String mm = "" + (calender.get(Calendar.MONTH) + 1);
        String dd = "" + calender.get(Calendar.DAY_OF_MONTH);
        String hh = "" + calender.get(Calendar.HOUR_OF_DAY);
        String mi = "" + calender.get(Calendar.MINUTE);
        String ss = "" + calender.get(Calendar.SECOND);

        yyyy = formatString(yyyy, 4);
        mm   = formatString(mm, 2);
        dd   = formatString(dd, 2);
        hh   = formatString(hh, 2);
        mi   = formatString(mi, 2);
        ss   = formatString(ss, 2);

        return "D" + yyyy + "-" + mm + "-" + dd + "T" + hh + mi + ss;
    }

    /**
     * Formatiere das String.
     * Falls die Lnge des Strings kleiner als formatLength ist, fge "(0)" for
     * dem String.
     *
     * @param sDatum Das zu formatierte String.
     * @param formatLength Die ntige Lnge des Strings.
     *
     * @return Das formatierte String.
     */
    private static String formatString(String sDatum, int formatLength) {
        int           l   = formatLength - sDatum.length();
        StringBuilder nul = new StringBuilder();

        for (int i = 1; i <= l; i++) {
            nul.append("0");
        }

        return nul.toString() + sDatum;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Gebe das aktuelle Datum als long zurck.
     * Der Wert der Millisekunden wird auf 0 gesetzt.
     *
     * @return Datum in Millisekunden.
     */
    public static long getAktuelleDateTime() {
        Calendar calender = new GregorianCalendar();

        calender.set(Calendar.MILLISECOND, 0);

        return calender.getTimeInMillis();
    }

    /**
     * Gibt die Anzahl der Monate im Interval
     * (wie viel Monate werden in diesem Interval betroffen).
     *
     *  Falls datumVon grer als datumBis, wird negative Zahl zurckgegeben.
     *
     * @param datumVon Anfang des Intervals.
     * @param datumBis  Ende des Intervals.
     * @return Anzahl der Monate im Interval.
     */
    public static int getAnzahlDerBetroffenenMonate(long datumVon, long datumBis) {
        int  anzahl     = 0;
        int  plus_minus = 1;
        long beginn;
        long ende;

        if (datumVon <= datumBis) {
            beginn = getMonatAnfang(datumVon);
            ende   = getMonatAnfang(datumBis);
        } else {
            beginn     = getMonatAnfang(datumBis);
            ende       = getMonatAnfang(datumVon);
            plus_minus = -1;
        }

        long datum = beginn;

        while (datum <= ende) {
            anzahl++;
            datum = getVerschobenenDateTime(datum, 1, 0);
        }

        return anzahl * plus_minus;
    }

    /**
     * TypUmwandlung GUI DateTime Komponente ins long.
     *
     * @param datum -
     *            GUI DateTime Komponente
     * @return Datum in Millisekunden.
     */
    public static long getDateTimeAsLong(DateTime datum) {
        Calendar calender = new GregorianCalendar(datum.getYear(), datum.getMonth(), datum.getDay(), datum.getHours(), datum.getMinutes(), datum.getSeconds());

        return calender.getTimeInMillis();
    }

    /**
     * TypUmwandlung GUI DateTime Komponente ins long. Datum wird aus zwei
     * Komponenten erstellt.
     *
     * @param datumDate -
     *            GUI DateTime Komponente, die Information ber "yy.mm.dd"
     *            enthlt.
     * @param datumZeit -
     *            GUI DateTime Komponente, die Information ber "hh:mm:ss"
     *            enthlt.
     * @return Datum in Millisekunden.
     */
    public static long getDateTimeAsLong(DateTime datumDate, DateTime datumZeit) {
        Calendar calender = new GregorianCalendar(datumDate.getYear(), datumDate.getMonth(), datumDate.getDay(), datumZeit.getHours(), datumZeit.getMinutes(), datumZeit.getSeconds());

        return calender.getTimeInMillis();
    }

    /**
     * Ermittle das Datum als long aus dem String.
     * Zeit wird dabei auf 0 gesetzt.
     *
     * @param datumText Datum als String "dd.mm.yyyy".
     *
     * @return Datum als long aus dem String mit der Zeit 0.
     */
    public static long getDatumAlsLong(String datumText) {
        return getDatumAlsLong(datumText, "00:00");
    }

    /**
     * Ermittle das Datum als long aus den Strings.
     * Sekunden und Millisekunden werden dabei auf 0 gesetzt.
     *
     * @param datumText Datum als String "dd.mm.yyyy".
     * @param zeitText Zeit als String "hh:mm".
     *
     * @return Datum als long aus den Strings.
     */
    public static long getDatumAlsLong(String datumText, String zeitText) {
        Calendar calender = new GregorianCalendar();

        try {
            int dd   = Integer.valueOf(datumText.substring(0, 2));
            int mm   = Integer.valueOf(datumText.substring(3, 5)) - 1;
            int yyyy = Integer.valueOf(datumText.substring(6));
            int hh   = Integer.valueOf(zeitText.substring(0, 2));
            int min  = Integer.valueOf(zeitText.substring(3, 5));

            calender.set(Calendar.MONTH, mm);
            calender.set(Calendar.YEAR, yyyy);
            calender.set(Calendar.DAY_OF_MONTH, dd);
            calender.setTimeInMillis(setzeZeit(calender.getTimeInMillis(), hh, min));
        }
        catch (Exception ignored) {}

        if (!getDatumZeitAlsString(calender.getTimeInMillis()).equals(datumText + "  " + zeitText + ":00")) {
            calender.setTimeInMillis(0);
        }

        return calender.getTimeInMillis();
    }

    /**
     * Stellt Datum als String dar.
     *
     * @param datumLong - Datum in Millisekunden.
     * @return Datum als String "dd.mm.yyyy".
     */
    public static String getDatumAlsString(long datumLong) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);

        String yyyy = "" + calender.get(Calendar.YEAR);

        // Monat+1, da Monat fngt von 0 an.
        String mm = "" + (calender.get(Calendar.MONTH) + 1);
        String dd = "" + calender.get(Calendar.DAY_OF_MONTH);

        yyyy = formatString(yyyy, 4);
        mm   = formatString(mm, 2);
        dd   = formatString(dd, 2);

        return dd + "." + mm + "." + yyyy;
    }

    /**
     * Stellt Datum als String dar.
     *
     * @param datumLong - Datum in Millisekunden.
     *
     * @return Datum als String "yyyy-mm-dd".
     */
    public static String getDatumInternationalAlsString(long datumLong) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);

        String yyyy = "" + calender.get(Calendar.YEAR);

        // Monat+1, da fngt von 0 Monat an.
        String mm = "" + (calender.get(Calendar.MONTH) + 1);
        String dd = "" + calender.get(Calendar.DAY_OF_MONTH);

        yyyy = formatString(yyyy, 4);
        mm   = formatString(mm, 2);
        dd   = formatString(dd, 2);

        return yyyy + "-" + mm + "-" + dd;
    }

    /**
     * Gebe das angegebene Datum mit der Zeit 0 zurck.
     *
     * @param datum Datum, deren Zeit auf 0 gesetzt werden soll.
     * @return Das angegebene Datum mit der Zeit 0 zurck.
     */
    public static long getDatumMitInitZeit(long datum) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datum);
        calender.set(Calendar.HOUR_OF_DAY, 0);
        calender.set(Calendar.MINUTE, 0);
        calender.set(Calendar.SECOND, 0);
        calender.set(Calendar.MILLISECOND, 0);

        return calender.getTimeInMillis();
    }

    /**
     * Stellt Datum und Uhrzeit als String dar.
     *
     * @param datumLong - Datum in Millisekunden.
     *
     * @return Datum als String "dd.mm.yyyy  hh:mm:ss".
     */
    public static String getDatumZeitAlsString(long datumLong) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);

        String yyyy = "" + calender.get(Calendar.YEAR);

        // Monat+1, da fngt von 0 Monat an.
        String mm = "" + (calender.get(Calendar.MONTH) + 1);
        String dd = "" + calender.get(Calendar.DAY_OF_MONTH);
        String hh = "" + calender.get(Calendar.HOUR_OF_DAY);
        String mi = "" + calender.get(Calendar.MINUTE);
        String ss = "" + calender.get(Calendar.SECOND);

        yyyy = formatString(yyyy, 4);
        mm   = formatString(mm, 2);
        dd   = formatString(dd, 2);
        hh   = formatString(hh, 2);
        mi   = formatString(mi, 2);
        ss   = formatString(ss, 2);

        return dd + "." + mm + "." + yyyy + "  " + hh + ":" + mi + ":" + ss;
    }

    /**
     * Stellt Datum und Uhrzeit als Zeitstempel dar.
     *
     * @param datumLong - Datum in Millisekunden.
     *
     * @return Datum als String "yyyymmddThhmm".
     */
    public static String getDatumZeitAlsZeitStempel(long datumLong) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);

        String yyyy = "" + calender.get(Calendar.YEAR);

        // Monat+1, da fngt von 0 Monat an.
        String mm = "" + (calender.get(Calendar.MONTH) + 1);
        String dd = "" + calender.get(Calendar.DAY_OF_MONTH);
        String hh = "" + calender.get(Calendar.HOUR_OF_DAY);
        String mi = "" + calender.get(Calendar.MINUTE);

        // String ss = "" + calender.get(Calendar.SECOND);
        yyyy = formatString(yyyy, 4);
        mm   = formatString(mm, 2);
        dd   = formatString(dd, 2);
        hh   = formatString(hh, 2);
        mi   = formatString(mi, 2);

        // ss   = formatString(ss, 2);
        return yyyy + mm + dd + "T" + hh + mi;
    }

    /**
     * Setze den Sekundenwert des angegebenen Datums auf 0 und
     * gebe das genderte Datum zurck.
     *
     * @param datum Datum, dessen Sekundenwert auf 0 gesetzt werden soll.
     * @return Das angegebene Datum mit dem Sekundenwert 0 zurck.
     */
    public static long getDatumZeitOhneSekunden(long datum) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datum);
        calender.set(Calendar.SECOND, 0);
        calender.set(Calendar.MILLISECOND, 0);

        return calender.getTimeInMillis();
    }

    /**
     * Stellt Datum und Uhrzeit ohne Sekunden als String dar.
     *
     * @param datumLong - Datum in Millisekunden.
     *
     * @return Datum als String "dd.mm.yyyy  hh:mm".
     */
    public static String getDatumZeitOhneSekundenAlsString(long datumLong) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);

        String yyyy = "" + calender.get(Calendar.YEAR);

        // Monat+1, da fngt von 0 Monat an.
        String mm = "" + (calender.get(Calendar.MONTH) + 1);
        String dd = "" + calender.get(Calendar.DAY_OF_MONTH);
        String hh = "" + calender.get(Calendar.HOUR_OF_DAY);
        String mi = "" + calender.get(Calendar.MINUTE);

        yyyy = formatString(yyyy, 4);
        mm   = formatString(mm, 2);
        dd   = formatString(dd, 2);
        hh   = formatString(hh, 2);
        mi   = formatString(mi, 2);

        return dd + "." + mm + "." + yyyy + "  " + hh + ":" + mi;
    }

    /**
     * Gebe das Aktuelle Datum mit der Zeit 0 zurck.
     *
     * @return Datum in Millisekunden.
     */
    public static long getInitDateTime() {
        return getDatumMitInitZeit(System.currentTimeMillis());
    }

    /**
     * Gebe das Aktuelle Datum mit der Zeit 23:59:00 zurck.
     *
     * @return Datum in Millisekunden.
     */
    public static long getInitDateTimeEnde() {
        Calendar calender = new GregorianCalendar();

        calender.set(Calendar.HOUR_OF_DAY, 23);
        calender.set(Calendar.MINUTE, 59);
        calender.set(Calendar.SECOND, 0);
        calender.set(Calendar.MILLISECOND, 0);

        return calender.getTimeInMillis();
    }

    /**
     * Gibt das Jahr zurck.
     * @param datumLong - Datum in Millisekunden.
     * @return Jahr fr das im Parameter angegebene Datum.
     */
    public static int getJahr(long datumLong) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);

        return calender.get(Calendar.YEAR);
    }

    /**
     * Gebe das angegebene Datum, das auf den Anfang des Jahres initialisiert wurde.
     *
     * @param datum Datum, deren Zeit, Tag und Monat auf den Anfang des Jahres initialisiert wurden.
     * @return Das angegebene Datum mit der Zeit 0, Tag 1 und Monat Januar zurck.
     */
    public static long getJahrAnfang(long datum) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datum);
        calender.set(Calendar.HOUR_OF_DAY, 0);
        calender.set(Calendar.MINUTE, 0);
        calender.set(Calendar.SECOND, 0);
        calender.set(Calendar.MILLISECOND, 0);
        calender.set(Calendar.DAY_OF_MONTH, 1);
        calender.set(Calendar.MONTH, 0);

        return calender.getTimeInMillis();
    }

    /**
     * Ermittle den maximalen Tag des im Parameter 'datum' angegebenen Monats.
     *
     * @param datum Datum, das den Monat liefert.
     *
     * @return Maximalen Tag des im Parameter 'datum' angegebenen Monats.
     */
    public static int getMaximalenTagImMonat(long datum) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datum);

        return calender.getActualMaximum(Calendar.DAY_OF_MONTH);
    }

    /**
     * Gibt die Nummer des Monats zurck (Januar = 1, ...).
     *
     * @param datum Datum in Millisekunden.
     * @return Nummer des Monats fr angegebenen Datum.
     */
    public static int getMonat(long datum) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datum);

        return calender.get(Calendar.MONTH) + 1;
    }

    /**
     * Gebe das angegebene Datum, das auf den Anfang des Monats initialisiert wurde.
     *
     * @param datum Datum, deren Zeit und Tag auf den Anfang des Monats initialisiert wurden.
     * @return Das angegebene Datum mit der Zeit 0 und Tag 1 zurck.
     */
    public static long getMonatAnfang(long datum) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datum);
        calender.set(Calendar.HOUR_OF_DAY, 0);
        calender.set(Calendar.MINUTE, 0);
        calender.set(Calendar.SECOND, 0);
        calender.set(Calendar.MILLISECOND, 0);
        calender.set(Calendar.DAY_OF_MONTH, 1);

        return calender.getTimeInMillis();
    }

    /**
     * Gebe das angegebene Datum, das auf das Ende des Monats mit der Zeit 23:59:00 initialisiert wurde.
     *
     * @param datum Datum, deren Zeit und Tag auf das Ende des Monats initialisiert wurden.
     * @return Das angegebene Datum mit der Zeit 23:59:00 und dem letzten Tag des Monats zurck.
     */
    public static long getMonatEnde(long datum) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datum);

        // Zeit:
        calender.set(Calendar.HOUR_OF_DAY, 23);
        calender.set(Calendar.MINUTE, 59);
        calender.set(Calendar.SECOND, 0);
        calender.set(Calendar.MILLISECOND, 0);

        // Tag:
        calender.set(Calendar.DAY_OF_MONTH, calender.getActualMaximum(Calendar.DAY_OF_MONTH));

        return calender.getTimeInMillis();
    }

    /**
     * Ermittle den Namen des Monats.
     *
     * @param monat Monat.
     * @return Den Namen des Monats.
     */
    public static String getMonatNamen(int monat) {
        switch (monat) {
        case 1 :
            return "Januar";

        case 2 :
            return "Februar";

        case 3 :
            return "Mrz";

        case 4 :
            return "April";

        case 5 :
            return "Mai";

        case 6 :
            return "Juni";

        case 7 :
            return "Juli";

        case 8 :
            return "August";

        case 9 :
            return "September";

        case 10 :
            return "Oktober";

        case 11 :
            return "November";

        case 12 :
            return "Dezember";

        default :
            return "";
        }
    }

    /**
     * Gibt vom angegebenen Datum ausgehend, den nchsten Tag zurck.
     * Die Zeit wird nicht gendert.
     *
     * @param datumLong Start-Datum.
     *
     * @return Datum mit dem nchsten Tag.
     */
    public static long getNaechstenTag(long datumLong) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);

        int monat = calender.get(Calendar.MONTH);
        int jahr  = calender.get(Calendar.YEAR);
        int tag   = calender.get(Calendar.DAY_OF_MONTH);

        // int stunde   = calender.get(Calendar.HOUR_OF_DAY);
        //
        // Falls das Datum das Jahresende darstellt (31.12),
        // nehme 01.01 des nchsten Jahres:
        if ((monat == 11) && (tag == 31)) {
            calender.set(Calendar.DAY_OF_MONTH, 1);
            calender.set(Calendar.MONTH, 0);
            calender.set(Calendar.YEAR, jahr + 1);
        } else {

            // Falls der Tag der letzte in diesem Monat ist,
            // nehme 01 Tag des nchsten Monats:
            if (tag == calender.getActualMaximum(Calendar.DAY_OF_MONTH)) {
                calender.set(Calendar.DAY_OF_MONTH, 1);
                calender.set(Calendar.MONTH, monat + 1);
            }

            // Sonst ndere nur den Tag:
            else {
                calender.set(Calendar.DAY_OF_MONTH, tag + 1);
            }
        }

        // Falls die Zeit umgestellt wird:
        // calender.set(Calendar.HOUR_OF_DAY, stunde);
        return calender.getTimeInMillis();
    }

    /**
     * Gibt die Stunden zurck.
     *
     * @param datum Datum in Millisekunden.
     * @return Stunden fr angegebenen Datum.
     */
    public static int getStunden(long datum) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datum);

        return calender.get(Calendar.HOUR_OF_DAY);
    }

    /**
     * Gibt den Tag zurck.
     *
     * @param datumLong - Datum in Millisekunden.
     * @return Tag fr das im Parameter angegebene Datum.
     */
    public static int getTag(long datumLong) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);

        return calender.get(Calendar.DAY_OF_MONTH);
    }

    /**
     * Stellt Datum als String dar.
     *
     * @param datumLong - Datum in Millisekunden.
     * @return Datum als String "hh:mm:ss".
     */
    public static String getUhrzeitAlsString(long datumLong) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);

        String hh = "" + calender.get(Calendar.HOUR_OF_DAY);
        String mi = "" + calender.get(Calendar.MINUTE);
        String ss = "" + calender.get(Calendar.SECOND);

        hh = formatString(hh, 2);
        mi = formatString(mi, 2);
        ss = formatString(ss, 2);

        return hh + ":" + mi + ":" + ss;
    }

    /**
     * Stellt Datum als String dar.
     *
     * @param datumLong - Datum in Millisekunden.
     * @return Datum als String "hh:mm".
     */
    public static String getUhrzeitAlsStringOhneSekunden(long datumLong) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);

        String hh = "" + calender.get(Calendar.HOUR_OF_DAY);
        String mi = "" + calender.get(Calendar.MINUTE);

        hh = formatString(hh, 2);
        mi = formatString(mi, 2);

        return hh + ":" + mi;
    }

    /**
     * Ermittelt die Uhrzeit des Datums in Millisekunden.
     *
     * @param datumLong - Datum in Millisekunden.
     *
     * @return Uhrzeit des Datums in Millisekunden.
     */
    public static int getUhrzeitInMillis(long datumLong) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);

        int hh = calender.get(Calendar.HOUR_OF_DAY);
        int mi = calender.get(Calendar.MINUTE);
        int ss = calender.get(Calendar.SECOND);
        int ms = calender.get(Calendar.MILLISECOND);

        return ((hh * 60 + mi) * 60 + ss) * 1000 + ms;
    }

    /**
     * Ermittelt die angegebene Uhrzeit in Millisekunden.
     *
     * @param hh Stunden.
     * @param mi Minuten.
     *
     * @return Die angegebene Uhrzeit in Millisekunden.
     */
    public static int getUhrzeitInMillis(int hh, int mi) {
        return ((hh * 60 + mi) * 60) * 1000;
    }

    /**
     * Gibt vom angegebenen Datum ausgehend, das Datum mit angegebenen
     * Verschiebung in Monaten und Tagen zurck.
     * Falls im Monat dieser Tag existiert nicht,
     * wird maximaler Tag des Monats gesetzt.
     * Die Zeit wird nicht gendert.
     *
     * @param datumLong Start-Datum.
     * @param verschiebungMonate Rckgabedatum wird um Anzahl der Monate gegenber angegebenem Datum verschoben.
     * @param verschiebungTage Rckgabedatum wird um Anzahl der Tage gegenber angegebenem Datum verschoben.
     *
     * @return Datum mit angegebenen Verschiebung in Monaten und Tagen
     */
    public static long getVerschobenenDateTime(long datumLong, int verschiebungMonate, int verschiebungTage) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);

        // Plus Monate:
        int monat    = calender.get(Calendar.MONTH);
        int jahr     = calender.get(Calendar.YEAR);
        int tag      = calender.get(Calendar.DAY_OF_MONTH);
        int stunde   = calender.get(Calendar.HOUR_OF_DAY);
        int jahrPlus = (monat + verschiebungMonate) / 12;
        int monatNeu = (monat + verschiebungMonate) - jahrPlus * 12;

        // Zuerst wird der Tag auf 1 gesetzt,
        // um den Fall, wenn im ntigen Monat wenige Tagen sind,
        // zu bercksichtigen:
        calender.set(Calendar.DAY_OF_MONTH, 1);
        calender.set(Calendar.MONTH, monatNeu);

        // Setze das Jahr:
        calender.set(Calendar.YEAR, jahr + jahrPlus);

        // Falls in diesem Monat so viel Tage nicht existieren,
        // setze max. mglichen Tag fr den Monat:
        if (tag > calender.getActualMaximum(Calendar.DAY_OF_MONTH)) {
            tag = calender.getActualMaximum(Calendar.DAY_OF_MONTH);
        }

        calender.set(Calendar.DAY_OF_MONTH, tag);

        // Plus Tage:
        calender.setTimeInMillis(calender.getTimeInMillis() + (ANZAHL_MILLISEKUNDEN_PRO_TAG * verschiebungTage));

        // Falls die Zeit umgestellt wird:
        calender.set(Calendar.HOUR_OF_DAY, stunde);

        return calender.getTimeInMillis();
    }

    /**
     * Gibt vom aktuellen Datum ausgehend, das Datum mit angegebenen
     * Verschiebung und der angegebenen Uhrzeit zurck.
     *
     * @param verschiebungTage Rckgabedatum wird um Anzahl Tage gegenber aktuellem Datum verschoben.
     * @param Stunde Rckgabedatum wird auf Stunde gesetzt.
     * @param Minute Rckgabedatum wird auf Minute gesetzt.
     * @param Sekunde Rckgabedatum wird auf Sekunde gesetzt.
     * @return Ergebnisdatum: Vom aktuellen Datum ausgehend, das Datum mit angegebenen
     * Verschiebung und der angegebenen Uhrzeit zurck.
     */
    public static long getVerschobenenDateTime(int verschiebungTage, int Stunde, int Minute, int Sekunde) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(calender.getTimeInMillis() + (ANZAHL_MILLISEKUNDEN_PRO_TAG * verschiebungTage));
        calender.set(Calendar.HOUR_OF_DAY, Stunde);
        calender.set(Calendar.MINUTE, Minute);
        calender.set(Calendar.SECOND, Sekunde);
        calender.set(Calendar.MILLISECOND, 0);

        return calender.getTimeInMillis();
    }

    /**
     * Ermittle rckwrts verschobenen Zeitpunkt.
     * Den angegebenen Zeitpunkt wird um angegebene Monate, Tage, Stunden nach hinten verschoben.
     *
     * @param datumLong Start-Datum.
     * @param minusMonate Anzahl der Monate fr die Verschiebung.
     * @param minusTage Anzahl der Tage fr die Verschiebung.
     * @param minusStunden Anzahl der Stunden fr die Verschiebung.
     *
     * @return Der rckwrts verschobene Zeitpunkt.
     */
    public static long getVerschobenenRueckwaertsZeitpunkt(long datumLong, int minusMonate, int minusTage, int minusStunden) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);

        int jahr   = calender.get(Calendar.YEAR);
        int monat  = calender.get(Calendar.MONTH);
        int tag    = calender.get(Calendar.DAY_OF_MONTH);
        int stunde = calender.get(Calendar.HOUR_OF_DAY);

        // Minus Monate:
        int minusJahre = 0;
        int monatNeu;

        if (monat >= minusMonate) {
            monatNeu = monat - minusMonate;
        } else {
            minusJahre = minusMonate / 12;
            monatNeu   = monat - (minusMonate - 12 * minusJahre);

            if (monatNeu < 0) {
                minusJahre++;
                monatNeu = 12 + monatNeu;
            }
        }

        // Zuerst wird der Tag auf 1 gesetzt,
        // um den Fall, wenn im ntigen Monat wenige Tagen sind,
        // zu bercksichtigen:
        calender.set(Calendar.DAY_OF_MONTH, 1);
        calender.set(Calendar.MONTH, monatNeu);

        // Setze das Jahr:
        calender.set(Calendar.YEAR, jahr - minusJahre);

        // Falls in diesem Monat so viel Tage nicht existieren,
        // setze max. mglichen Tag fr den Monat:
        if (tag > calender.getActualMaximum(Calendar.DAY_OF_MONTH)) {
            tag = calender.getActualMaximum(Calendar.DAY_OF_MONTH);
        }

        calender.set(Calendar.DAY_OF_MONTH, tag);

        // Minus Tage:
        calender.setTimeInMillis(calender.getTimeInMillis() - (DatumFunktionen.ANZAHL_MILLISEKUNDEN_PRO_TAG * minusTage));

        // Falls die Zeit umgestellt wird:
        calender.set(Calendar.HOUR_OF_DAY, stunde);

        // Minus Stunden:
        calender.setTimeInMillis(calender.getTimeInMillis() - (1000 * 60 * 60 * minusStunden));

        return calender.getTimeInMillis();
    }

    /**
     * Gibt die Nummer des WochenTags zurck
     * (Montag hat die Nummer 1 ... Sonntag - 7).
     *
     * @param datumLong - Datum in Millisekunden.
     *
     * @return Nummer des WochenTags.
     */
    public static int getWochenTag(long datumLong) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);

        int wochenTag = calender.get(Calendar.DAY_OF_WEEK) - 1;

        if (wochenTag < 1) {
            wochenTag = 7;
        }

        return wochenTag;
    }

    /**
     * Stellt die Zeitdauer als String dar.
     *
     * @param zeitDauer Zeitdauer, die als String dargestellt werden soll.
     *
     * @return Zeitdauer als String "hh:mm:ss".
     */
    public static String getZeitdauerAlsString(long zeitDauer) {
        long  zeitHr  = zeitDauer / (1000 * 60 * 60);
        long  zeitMin = (zeitDauer - zeitHr * 60 * 60 * 1000) / (1000 * 60);
        float zeitSek = (zeitDauer - zeitHr * 60 * 60 * 1000 - zeitMin * 60 * 1000) / 1000f;

        return String.format("%02d:%02d:%02d", zeitHr, zeitMin, Math.round(zeitSek));
    }

    /**
     * Stellt die Zeitdauer ohne Millisekunden als String dar.
     *
     * @param zeitDauer Zeitdauer, die als String dargestellt werden soll.
     *
     * @return Zeitdauer als String "hh:mm".
     */
    public static String getZeitdauerOhneMillisAlsString(long zeitDauer) {
        String zd = getZeitdauerAlsString(zeitDauer);

        return zd.substring(0, zd.length() - 3);
    }

    //~ METHODEN ==============================================================

    /**
     * Initialisiere GUI DateTime Komponente mit dem Datum 01.01.2009 01:01:01.
     * WICHTIG_: Ntig ist, wenn man das Datum ndern wird um Nebeneffekte zu vermeiden.
     * (z.B. falls Datum 31.01.2009 ist und soll auf 02.02.2009 eingestellt werden,
     * wird beim ndern des Monats auf Februar Monat Januar bleiben, da es kein 31.02 gibt)
     *
     * @param datum Die zu initialisierte GUI DateTime Komponente.
     */
    private static void initDateTime(DateTime datum) {
        datum.setYear(2009);
        datum.setMonth(0);
        datum.setDay(1);
        datum.setHours(1);
        datum.setMinutes(1);
        datum.setSeconds(1);
    }

    /**
     * Stellt fest, ob das angegebene Intervall nicht lnger als 24 Stunden ist.
     * Dabei wird die Sommer/Winter-Zeitumstellung bercksichtigt:
     * also das Intervall soll im Bereich von xx Uhr bis xx Uhr des nchsten Tags liegen.
     *
     * @param anfang Anfang des Intervalls.
     * @param ende Ende des Intervalls.
     *
     * @return true, falls das angegebene Intervall nicht lnger als 24 Stunden ist,
     * falls sonst.
     */
    public static boolean istIntervallNichtLaengerAls24Stunden(long anfang, long ende) {
        return (ende < getNaechstenTag(anfang));
    }

    /**
     * Kopiere die Zeit fr den Datum.
     *
     * @param datum Datum, dessen Zeit gendert werden soll.
     * @param datumMuster Datum, dessen Zeitwert kopiert wird.
     * @return Neuen Datum-Wert mit der genderten (aus dem anderen Datum kopierten) Zeit.
     */
    public static long kopiereZeit(long datum, long datumMuster) {

        // Datum, das gendert werden soll:
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datum);

        // Datum, das neue Zeit-Werte enthlt:
        Calendar calenderMuster = new GregorianCalendar();

        calenderMuster.setTimeInMillis(datumMuster);

        // Setze die Zeit:
        calender.set(Calendar.HOUR_OF_DAY, calenderMuster.get(Calendar.HOUR_OF_DAY));
        calender.set(Calendar.MINUTE, calenderMuster.get(Calendar.MINUTE));
        calender.set(Calendar.SECOND, calenderMuster.get(Calendar.SECOND));
        calender.set(Calendar.MILLISECOND, calenderMuster.get(Calendar.MILLISECOND));

        //
        return calender.getTimeInMillis();
    }

    /**
     * Initialisiere GUI DateTime Komponente mit dem long Wert (Millisekunden).
     *
     * @param datum -
     *            GUI DateTime Komponente.
     * @param datumLong -
     *            Anzahl der Millisekunden.
     */
    public static void setzeDateTime(DateTime datum, long datumLong) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datumLong);
        initDateTime(datum);
        datum.setYear(calender.get(Calendar.YEAR));
        datum.setMonth(calender.get(Calendar.MONTH));
        datum.setDay(calender.get(Calendar.DAY_OF_MONTH));
        datum.setHours(calender.get(Calendar.HOUR_OF_DAY));
        datum.setMinutes(calender.get(Calendar.MINUTE));
        datum.setSeconds(calender.get(Calendar.SECOND));
    }

    /**
     * Initialisiere Array von GUI DateTime Komponenten mit dem long Wert
     * (Millisekunden)
     *
     * @param arDatum -
     *            Array DateTime[4] (arDatum[0]-Datum von, arDatum[1]-Zeit von,
     *            arDatum[2]-Datum bis, arDatum[3]-Zeit bis).
     * @param arLong -
     *            Array long[2] (arLong[0]-DatumVon in Millisekunden,
     *            arLong[1]-DatumBis in Millisekunden).
     */
    @SuppressWarnings({ "UnusedDeclaration" })
    public static void setzeDateTimeVonBis(DateTime[] arDatum, long[] arLong) {
        if ((arDatum.length != 4) || (arLong.length < 2)) {

            // noinspection ThrowableInstanceNeverThrown
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, new Throwable("Falsche Parameter in der Funktion setzeDateTimeVonBis() (arDatum muss Lnge 4 und arLong Lnge >=2 haben!)"));
        } else {
            setzeDateTime(arDatum[0], arLong[0]);
            setzeDateTime(arDatum[1], arLong[0]);
            setzeDateTime(arDatum[2], arLong[1]);
            setzeDateTime(arDatum[3], arLong[1]);
        }
    }

    /**
     * Setze die Zeit fr das angegebene Datum und gebe das genderte Datum zurck.
     * Sekunden und Millisekunden werden dabei auf 0 gesetzt.
     *
     * @param datum Datum, deren Zeit gesetzt werden soll.
     * @param st Stunde.
     * @param min Minuten.
     *
     * @return Datum mit der neu gesetzten Zeit.
     * Falls die Zeit falsch angegeben wird, wird die nicht gendert.
     */
    public static long setzeZeit(long datum, int st, int min) {
        Calendar calender = new GregorianCalendar();

        calender.setTimeInMillis(datum);

        if ((st >= 0) && (st < 25) && (min >= 0) && (min < 60)) {
            calender.setTimeInMillis(datum);
            calender.set(Calendar.HOUR_OF_DAY, st);
            calender.set(Calendar.MINUTE, min);
            calender.set(Calendar.SECOND, 0);
            calender.set(Calendar.MILLISECOND, 0);
        } else {
            calender.setTimeInMillis(datum);
        }

        return calender.getTimeInMillis();
    }

    /**
     * Aus DateTime Array (entspricht Datum von, Datum bis) erzeuge long Array
     * (entspricht Datum von, Datum bis in Millisekunden.).
     *
     * @param arLong -
     *            Array long[2] (arLong[0]-DatumVon in Millisekunden,
     *            arLong[1]-DatumBis in Millisekunden).
     * @param arDatum -
     *            Array DateTime[4] (arDatum[0]-Datum von, arDatum[1]-Zeit von,
     *            arDatum[2]-Datum bis, arDatum[3]-Zeit bis).
     */
    public static void speichereDateTimeVonBis(long[] arLong, DateTime[] arDatum) {
        if ((arDatum.length != 4) || (arLong.length < 2)) {

            // noinspection ThrowableInstanceNeverThrown
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, new Throwable("Falsche Parameter in der Funktion speichereDateTimeVonBis (arDatum muss Lnge 4 und arLong Lnge >=2 haben!)"));
        } else {
            arLong[0] = getDateTimeAsLong(arDatum[0], arDatum[1]);
            arLong[1] = getDateTimeAsLong(arDatum[2], arDatum[3]);
        }
    }

    /**
     * Stellt fest, ob sich die Intervalle berschneiden.
     *
     * @param von1 Anfang des ersten Intervalls.
     * @param bis1 Ende des ersten Intervalls.
     * @param von2 Anfang des zweiten Intervalls.
     * @param bis2 Ende des zweiten Intervalls.
     *
     * @return true, falls sich die Intervalle berschneiden,
     * false - sonst.
     */
    public static boolean ueberschneidenSichDieIntervalle(long von1, long bis1, long von2, long bis2) {

        // Die Intervalle berschneiden sich, falls das 2.Intervall fngt vor dem Ende des 1.Intervall an
        // und endet nach dem Anfang des 1.Intervall:
        return (von2 < bis1) && (bis2 > von1);
    }

    /**
     * Vergleiche die Uhrzeit der zwei Daten.
     *
     * @param datum1 Datum 1.
     * @param datum2 Datum 2.
     *
     * @return Negative Zahl - falls die Uhrzeit vom Datum 1 kleine als die Uhrzeit vom Datum 2 ist,
     * 0 - falls die gleich sind und
     * positive Zahl - falls Uhrzeit vom Datum 1 grer als die Uhrzeit vom Datum 2 ist.
     */
    public static int vergleicheDieUhrzeit(long datum1, long datum2) {
        Integer zeit1 = getUhrzeitInMillis(datum1);
        int     zeit2 = getUhrzeitInMillis(datum2);

        return zeit1.compareTo(zeit2);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 14.02.12
