/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.archiv;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKindCombination;
import de.bsvrz.dav.daf.main.archive.ArchiveOrder;
import de.bsvrz.dav.daf.main.archive.ArchiveQueryPriority;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestOption;
import de.bsvrz.dav.daf.main.archive.TimingType;

import de.inovat.buv.inovat.lib.funktionen.datum.DatumFunktionen;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Die Klasse reprsentiert die Daten bei einer Archivanfrage.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class ArchivanfrageDaten {

    /**
     * Art der Archivanfrage:
     * <ul>
     * <li><b>aktuelle Daten</b>,
     * <li>nachgelieferte Daten,
     * <li>nachgefordert-aktuelle Daten,
     * <li>nachgefordert-nachgelieferte Daten.
     * </ul>
     */
    private boolean _aktuelleDaten;

    /**
     * Anzahl der Datenstze, die die Archivanfrage begrenzt.
     * Wert {@link #_istDatenanzahlBegrenzt} = true.
     */
    private int _anzahlDaten;

    /**
     * Bis Datum der Archivanfrage bei der absoluten Zeitbereich-Angaben
     * ({@link #_istAbsoluteZeitangabe} = true).
     * Zeitbereichangabe ist dabei nicht gleich {@link ZeitBereichDerAnfrage#Datenindex}.
     */
    private long _bisDatum;

    /**
     * Bis Index der Archivanfrage bei der absoluten Zeitbereich-Angaben
     * ({@link #_istAbsoluteZeitangabe} = true).
     * Zeitbereichangabe ist dabei gleich {@link ZeitBereichDerAnfrage#Datenindex}.
     */
    private int _bisIndex;

    /**
     * Anzahl der Monate fr die relative Zeitbereichangabe.
     * ({@link #_istAbsoluteZeitangabe} = false).
     */
    private int _bisMonate;

    /**
     * Anzahl der Stunden fr die relative Zeitbereichangabe.
     * ({@link #_istAbsoluteZeitangabe} = false).
     */
    private int _bisStunden;

    /**
     * Anzahl der Tage fr die relative Zeitbereichangabe.
     * ({@link #_istAbsoluteZeitangabe} = false).
     */
    private int _bisTage;

    /**
     * Stellt fest, ob der Zeitbereich absolut angegeben wird.
     * <li>ja:          Bereichangabe von Datum/Index bis Datum/Index</li>
     * <li>nein:        von/bis  Anzahl der Monaten/Tagen/Stunden vor dem aktuellen Zeitpunkt</li>.
     */
    private boolean _istAbsoluteZeitangabe;

    /**
     * Stellt fest, ob die relative Zeitangebe ({@link #_istAbsoluteZeitangabe} = false) aufgerudet werden soll.
     */
    private boolean _istBisAufrunden;

    /**
     * Stellt fest, ob der Zeitbereich durch die Anzahl der Daten begrenzt wird.
     * Falls ja, werden die Werte von Datum/Index nicht bercksichtigt.
     */
    private boolean _istDatenanzahlBegrenzt;

    /**
     * Stellt fest, ob die relative Zeitangebe ({@link #_istAbsoluteZeitangabe} = false) aufgerudet werden soll.
     */
    private boolean _istVonAufrunden;

    /**
     * Definiert die Anfrageart (s. {@link Art}).
     * (wird nicht umbennant, um die gespeicherte Einstellungen bernehmen zu knnen)
     */
    private String _isZustandsAnfrage;

    /**
     * Art der Archivanfrage:
     * <ul>
     * <li>aktuelle Daten,
     * <li>nachgelieferte Daten,
     * <li><b>nachgefordert-aktuelle Daten</b>,
     * <li>nachgefordert-nachgelieferte Daten.
     * </ul>
     */
    private boolean _nachgefordertAktuelleDaten;

    /**
     * Art der Archivanfrage:
     * <ul>
     * <li>aktuelle Daten,
     * <li>nachgelieferte Daten,
     * <li>nachgefordert-aktuelle Daten,
     * <li><b>nachgefordert-nachgelieferte Daten</b>.
     * </ul>
     */
    private boolean _nachgefordertNachgelieferteDaten;

    /**
     * Art der Archivanfrage:
     * <ul>
     * <li>aktuelle Daten,
     * <li><b>nachgelieferte Daten</b>,
     * <li>nachgefordert-aktuelle Daten,
     * <li>nachgefordert-nachgelieferte Daten.
     * </ul>
     */
    private boolean _nachgelieferteDaten;

    /** Gibt die Prioritt der Anfrage an (s. {@link Prioritaet}). */
    private String _prioritaetDerAnfrage;

    /** Sortierreihenfolge bei der Archivanfrage (s.{@link Sortierung}). */
    private String _sortierReihenfolge;

    /**
     * Von Datum der Archivanfrage bei der absoluten Zeitbereich-Angaben
     * ({@link #_istAbsoluteZeitangabe} = true).
     * Zeitbereichangabe ist dabei nicht gleich {@link ZeitBereichDerAnfrage#Datenindex}.
     */
    private long _vonDatum;

    /**
     * Von Index der Archivanfrage bei der absoluten Zeitbereich-Angaben
     * ({@link #_istAbsoluteZeitangabe} = true).
     * Zeitbereichangabe ist dabei gleich {@link ZeitBereichDerAnfrage#Datenindex}.
     */
    private int _vonIndex;

    /**
     * Anzahl der Monate fr die relative Zeitbereichangabe.
     * ({@link #_istAbsoluteZeitangabe} = false).
     */
    private int _vonMonate;

    /**
     * Anzahl der Stunden fr die relative Zeitbereichangabe.
     * ({@link #_istAbsoluteZeitangabe} = false).
     */
    private int _vonStunden;

    /**
     * Anzahl der Tage fr die relative Zeitbereichangabe.
     * ({@link #_istAbsoluteZeitangabe} = false).
     */
    private int _vonTage;

    /** Gibt den Zeitbereich der Archivanfrage an (s. {@link ZeitBereichDerAnfrage}). */
    private String _zeitBereichDerAnfrage;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse ArchivanfrageDaten.
     */
    public ArchivanfrageDaten() {

        //
        this(Art.Zustandsanfrage.name(), Sortierung.Datenindex.name(), Prioritaet.Hoch.name(),  //
             true, false,                                                                       //
             false, false,                                                                      //
             ZeitBereichDerAnfrage.Datenzeitstempel.name(),                                     //
             0, 0,                                                                              //
             false, 0);

        // Initialisiere von/bisDatum:
        long jetzt = DatumFunktionen.getAktuelleDateTime();

        jetzt     = DatumFunktionen.getDatumZeitOhneSekunden(jetzt);
        _vonDatum = DatumFunktionen.getMonatAnfang(jetzt);
        _bisDatum = jetzt;
    }

    /**
     * Erzeugt eine Instanz der Klasse ArchivanfrageDaten
     * fr {@link ZeitBereichDerAnfrage#Datenindex}.
     * Absoluter Zeitbereich ist ber von/bis Index definiert.
     *
     * @param isZustandsAnfrage Siehe {@link #_isZustandsAnfrage}
     * @param sortierReihenfolge Siehe {@link #_sortierReihenfolge}
     * @param prioritaetDerAnfrage Siehe {@link #_prioritaetDerAnfrage}
     * @param aktuelleDaten Siehe {@link #_aktuelleDaten}
     * @param nachgelieferteDaten Siehe {@link #_nachgelieferteDaten}
     * @param nachgefordertAktuelleDaten Siehe {@link #_nachgefordertAktuelleDaten}
     * @param nachgefordertNachgelieferteDaten Siehe {@link #_nachgefordertNachgelieferteDaten}
     *
     * @param vonIndex Siehe {@link #_vonIndex}
     * @param bisIndex Siehe {@link #_bisIndex}
     * @param istDatenanzahlBegrenzt Siehe {@link #_istDatenanzahlBegrenzt}
     * @param anzahlDaten Siehe {@link #_anzahlDaten}
     */
    public ArchivanfrageDaten(String isZustandsAnfrage, String sortierReihenfolge, String prioritaetDerAnfrage,  //
                              boolean aktuelleDaten, boolean nachgelieferteDaten,                                //
                              boolean nachgefordertAktuelleDaten, boolean nachgefordertNachgelieferteDaten,      //
                              int vonIndex, int bisIndex,                                                        //
                              boolean istDatenanzahlBegrenzt, int anzahlDaten) {

        //
        this(isZustandsAnfrage, sortierReihenfolge, prioritaetDerAnfrage,   //
             aktuelleDaten, nachgelieferteDaten,                            //
             nachgefordertAktuelleDaten, nachgefordertNachgelieferteDaten,  //
             ZeitBereichDerAnfrage.Datenindex.name(), true,                 //
             0, 0, vonIndex, bisIndex,                                      //
             istDatenanzahlBegrenzt, anzahlDaten,                           //
             0, 0, 0, false,                                                //
             0, 0, 0, false);
    }

    /**
     * Erzeugt eine Instanz der Klasse ArchivanfrageDaten
     * fr {@link ZeitBereichDerAnfrage#Datenzeitstempel} oder {@link ZeitBereichDerAnfrage#Archivzeitstempel}.
     * Absoluter Zeitbereich ist ber von/bis Datum definiert.
     *
     * @param isZustandsAnfrage Siehe {@link #_isZustandsAnfrage}
     * @param sortierReihenfolge Siehe {@link #_sortierReihenfolge}
     * @param prioritaetDerAnfrage Siehe {@link #_prioritaetDerAnfrage}
     * @param aktuelleDaten Siehe {@link #_aktuelleDaten}
     * @param nachgelieferteDaten Siehe {@link #_nachgelieferteDaten}
     * @param nachgefordertAktuelleDaten Siehe {@link #_nachgefordertAktuelleDaten}
     * @param nachgefordertNachgelieferteDaten Siehe {@link #_nachgefordertNachgelieferteDaten}
     *
     * @param zeitBereichDerAnfrage Siehe {@link #_zeitBereichDerAnfrage}.
     *                  Mgliche Werte: {@link ZeitBereichDerAnfrage#Datenzeitstempel} oder {@link ZeitBereichDerAnfrage#Archivzeitstempel}.
     *
     * @param vonDatum Siehe {@link #_vonDatum}
     * @param bisDatum Siehe {@link #_bisDatum}
     *
     * @param istDatenanzahlBegrenzt Siehe {@link #_istDatenanzahlBegrenzt}
     * @param anzahlDaten Siehe {@link #_anzahlDaten}
     *
     *
     */
    public ArchivanfrageDaten(String isZustandsAnfrage, String sortierReihenfolge, String prioritaetDerAnfrage,  //
                              boolean aktuelleDaten, boolean nachgelieferteDaten,                                //
                              boolean nachgefordertAktuelleDaten, boolean nachgefordertNachgelieferteDaten,      //
                              String zeitBereichDerAnfrage,                                                      //
                              long vonDatum, long bisDatum,                                                      //
                              boolean istDatenanzahlBegrenzt, int anzahlDaten) {

        //
        this(isZustandsAnfrage, sortierReihenfolge, prioritaetDerAnfrage,   //
             aktuelleDaten, nachgelieferteDaten,                            //
             nachgefordertAktuelleDaten, nachgefordertNachgelieferteDaten,  //
             zeitBereichDerAnfrage, true,                                   //
             vonDatum, bisDatum, 0, 0,                                      //
             istDatenanzahlBegrenzt, anzahlDaten,                           //
             0, 0, 0, false,                                                //
             0, 0, 0, false);
    }

    /**
     * Erzeugt eine Instanz der Klasse ArchivanfrageDaten
     * fr {@link ZeitBereichDerAnfrage#Datenzeitstempel} oder {@link ZeitBereichDerAnfrage#Archivzeitstempel}.
     * Relativer Zeitbereich ist ber von/bis Monate/Tage/Stunden definiert.
     *
     * @param isZustandsAnfrage Siehe {@link #_isZustandsAnfrage}
     * @param sortierReihenfolge Siehe {@link #_sortierReihenfolge}
     * @param prioritaetDerAnfrage Siehe {@link #_prioritaetDerAnfrage}
     * @param aktuelleDaten Siehe {@link #_aktuelleDaten}
     * @param nachgelieferteDaten Siehe {@link #_nachgelieferteDaten}
     * @param nachgefordertAktuelleDaten Siehe {@link #_nachgefordertAktuelleDaten}
     * @param nachgefordertNachgelieferteDaten Siehe {@link #_nachgefordertNachgelieferteDaten}
     *
     * @param zeitBereichDerAnfrage Siehe {@link #_zeitBereichDerAnfrage}.
     *                  Mgliche Werte: {@link ZeitBereichDerAnfrage#Datenzeitstempel} oder {@link ZeitBereichDerAnfrage#Archivzeitstempel}.
     *
     * @param vonMonate Siehe {@link #_vonMonate}
     * @param vonTage Siehe {@link #_vonTage}
     * @param vonStunden Siehe {@link #_vonStunden}
     * @param istVonAufrunden Siehe {@link #_istVonAufrunden}
     * @param bisMonate Siehe {@link #_bisMonate}
     * @param bisTage Siehe {@link #_bisTage}
     * @param bisStunden Siehe {@link #_bisStunden}
     * @param istBisAufrunden Siehe {@link #_istBisAufrunden}
     */
    public ArchivanfrageDaten(String isZustandsAnfrage, String sortierReihenfolge, String prioritaetDerAnfrage,  //
                              boolean aktuelleDaten, boolean nachgelieferteDaten,                                //
                              boolean nachgefordertAktuelleDaten, boolean nachgefordertNachgelieferteDaten,      //
                              String zeitBereichDerAnfrage,                                                      //
                              int vonMonate, int vonTage, int vonStunden, boolean istVonAufrunden,               //
                              int bisMonate, int bisTage, int bisStunden, boolean istBisAufrunden) {

        //
        this(isZustandsAnfrage, sortierReihenfolge, prioritaetDerAnfrage,   //
             aktuelleDaten, nachgelieferteDaten,                            //
             nachgefordertAktuelleDaten, nachgefordertNachgelieferteDaten,  //
             zeitBereichDerAnfrage, false,                                  //
             0, 0, 0, 0,                                                    //
             false, 0,                                                      //
             vonMonate, vonTage, vonStunden, istVonAufrunden,               //
             bisMonate, bisTage, bisStunden, istBisAufrunden);
    }

    /**
     * Erzeugt eine Instanz der Klasse ArchivanfrageDaten.
     *
     * @param isZustandsAnfrage Siehe {@link #_isZustandsAnfrage}
     * @param sortierReihenfolge Siehe {@link #_sortierReihenfolge}
     * @param prioritaetDerAnfrage Siehe {@link #_prioritaetDerAnfrage}
     * @param aktuelleDaten Siehe {@link #_aktuelleDaten}
     * @param nachgelieferteDaten Siehe {@link #_nachgelieferteDaten}
     * @param nachgefordertAktuelleDaten Siehe {@link #_nachgefordertAktuelleDaten}
     * @param nachgefordertNachgelieferteDaten Siehe {@link #_nachgefordertNachgelieferteDaten}
     * @param zeitBereichDerAnfrage Siehe {@link #_zeitBereichDerAnfrage}
     * @param istAbsoluteZeitangabe Siehe {@link #_istAbsoluteZeitangabe}
     * @param vonDatum Siehe {@link #_vonDatum}
     * @param bisDatum Siehe {@link #_bisDatum}
     * @param vonIndex Siehe {@link #_vonIndex}
     * @param bisIndex Siehe {@link #_bisIndex}
     * @param istDatenanzahlBegrenzt Siehe {@link #_istDatenanzahlBegrenzt}
     * @param anzahlDaten Siehe {@link #_anzahlDaten}
     * @param vonMonate Siehe {@link #_vonMonate}
     * @param vonTage Siehe {@link #_vonTage}
     * @param vonStunden Siehe {@link #_vonStunden}
     * @param istVonAufrunden Siehe {@link #_istVonAufrunden}
     * @param bisMonate Siehe {@link #_bisMonate}
     * @param bisTage Siehe {@link #_bisTage}
     * @param bisStunden Siehe {@link #_bisStunden}
     * @param istBisAufrunden Siehe {@link #_istBisAufrunden}
     */
    private ArchivanfrageDaten(String isZustandsAnfrage, String sortierReihenfolge, String prioritaetDerAnfrage,  //
                               boolean aktuelleDaten, boolean nachgelieferteDaten,                                //
                               boolean nachgefordertAktuelleDaten, boolean nachgefordertNachgelieferteDaten,      //
                               String zeitBereichDerAnfrage, boolean istAbsoluteZeitangabe,                       //
                               long vonDatum, long bisDatum, int vonIndex, int bisIndex,                          //
                               boolean istDatenanzahlBegrenzt, int anzahlDaten,                                   //
                               int vonMonate, int vonTage, int vonStunden, boolean istVonAufrunden,               //
                               int bisMonate, int bisTage, int bisStunden, boolean istBisAufrunden) {

        //
        _isZustandsAnfrage    = isZustandsAnfrage;
        _sortierReihenfolge   = sortierReihenfolge;
        _prioritaetDerAnfrage = prioritaetDerAnfrage;

        //
        _aktuelleDaten                    = aktuelleDaten;
        _nachgelieferteDaten              = nachgelieferteDaten;
        _nachgefordertAktuelleDaten       = nachgefordertAktuelleDaten;
        _nachgefordertNachgelieferteDaten = nachgefordertNachgelieferteDaten;

        //
        _zeitBereichDerAnfrage = zeitBereichDerAnfrage;
        _istAbsoluteZeitangabe = istAbsoluteZeitangabe;

        //
        _vonDatum               = vonDatum;
        _bisDatum               = bisDatum;
        _vonIndex               = vonIndex;
        _bisIndex               = bisIndex;
        _istDatenanzahlBegrenzt = istDatenanzahlBegrenzt;
        _anzahlDaten            = anzahlDaten;

        //
        _vonMonate       = vonMonate;
        _vonTage         = vonTage;
        _vonStunden      = vonStunden;
        _istVonAufrunden = istVonAufrunden;
        _bisMonate       = bisMonate;
        _bisTage         = bisTage;
        _bisStunden      = bisStunden;
        _istBisAufrunden = istBisAufrunden;
    }

    //~ AUFZAEHLUNGSTYPEN =====================================================

    /** Art der Archivanfrage. */
    public static enum Art {
        Zustandsanfrage, Deltaanfrage;

        public static String[] ermittleElemente() {
            String[] ar = new String[Art.values().length];
            int      i  = 0;

            for (Art el : Art.values()) {
                ar[i++] = el.name();
            }

            return ar;
        }
    }

    /** Prioritt der Archivanfrage. */
    public static enum Prioritaet {
        Hoch, Mittel, Niedrig;

        public static String[] ermittleElemente() {
            String[] ar = new String[Prioritaet.values().length];
            int      i  = 0;

            for (Prioritaet el : Prioritaet.values()) {
                ar[i++] = el.name();
            }

            return ar;
        }
    }

    /** Sortierung der Archivanfrage. */
    public static enum Sortierung {
        Datenindex, Datenzeitstempel;

        public static String[] ermittleElemente() {
            String[] ar = new String[Sortierung.values().length];
            int      i  = 0;

            for (Sortierung el : Sortierung.values()) {
                ar[i++] = el.name();
            }

            return ar;
        }
    }

    /** Zeitbereich der Anfrage. */
    public static enum ZeitBereichDerAnfrage {
        Datenzeitstempel, Archivzeitstempel, Datenindex;

        public static String[] ermittleElemente() {
            String[] ar = new String[ZeitBereichDerAnfrage.values().length];
            int      i  = 0;

            for (ZeitBereichDerAnfrage el : ZeitBereichDerAnfrage.values()) {
                ar[i++] = el.name();
            }

            return ar;
        }
    }

    //~ METHODEN ==============================================================

    /**
     * Soll das Ende des Zeitbereichs ausgeschlossen sein.
     *
     * (Bei der Zeitbereichangabe [von Datum, bis Datum]
     * soll bis Datum um 1 Millisekunde verkleinert werden)
     *
     * @return true, falls das Ende des Zeitbereichs ausgeschlossen sein,
     * false - sonst.
     */
    public boolean bisZeitbereichAusschliessen() {
        if ((!_istDatenanzahlBegrenzt) && (!(_zeitBereichDerAnfrage.equals(ZeitBereichDerAnfrage.Datenindex.name())))) {
            return true;
        } else {
            return false;
        }
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if (obj == null) {
            return false;
        }

        if (getClass() != obj.getClass()) {
            return false;
        }

        ArchivanfrageDaten other = (ArchivanfrageDaten) obj;

        if (_aktuelleDaten != other._aktuelleDaten) {
            return false;
        }

        if (_anzahlDaten != other._anzahlDaten) {
            return false;
        }

        if (_bisDatum != other._bisDatum) {
            return false;
        }

        if (_bisIndex != other._bisIndex) {
            return false;
        }

        if (_bisMonate != other._bisMonate) {
            return false;
        }

        if (_bisStunden != other._bisStunden) {
            return false;
        }

        if (_bisTage != other._bisTage) {
            return false;
        }

        if (_isZustandsAnfrage == null) {
            if (other._isZustandsAnfrage != null) {
                return false;
            }
        } else if (!_isZustandsAnfrage.equals(other._isZustandsAnfrage)) {
            return false;
        }

        if (_istAbsoluteZeitangabe != other._istAbsoluteZeitangabe) {
            return false;
        }

        if (_istBisAufrunden != other._istBisAufrunden) {
            return false;
        }

        if (_istDatenanzahlBegrenzt != other._istDatenanzahlBegrenzt) {
            return false;
        }

        if (_istVonAufrunden != other._istVonAufrunden) {
            return false;
        }

        if (_nachgefordertAktuelleDaten != other._nachgefordertAktuelleDaten) {
            return false;
        }

        if (_nachgefordertNachgelieferteDaten != other._nachgefordertNachgelieferteDaten) {
            return false;
        }

        if (_nachgelieferteDaten != other._nachgelieferteDaten) {
            return false;
        }

        if (_prioritaetDerAnfrage == null) {
            if (other._prioritaetDerAnfrage != null) {
                return false;
            }
        } else if (!_prioritaetDerAnfrage.equals(other._prioritaetDerAnfrage)) {
            return false;
        }

        if (_sortierReihenfolge == null) {
            if (other._sortierReihenfolge != null) {
                return false;
            }
        } else if (!_sortierReihenfolge.equals(other._sortierReihenfolge)) {
            return false;
        }

        if (_vonDatum != other._vonDatum) {
            return false;
        }

        if (_vonIndex != other._vonIndex) {
            return false;
        }

        if (_vonMonate != other._vonMonate) {
            return false;
        }

        if (_vonStunden != other._vonStunden) {
            return false;
        }

        if (_vonTage != other._vonTage) {
            return false;
        }

        if (_zeitBereichDerAnfrage == null) {
            if (other._zeitBereichDerAnfrage != null) {
                return false;
            }
        } else if (!_zeitBereichDerAnfrage.equals(other._zeitBereichDerAnfrage)) {
            return false;
        }

        return true;
    }

    /**
     * Ermittle DAV-Art fr die Archivanfrage.
     *
     * @return DAV-Art fr die Archivanfrage.
     */
    public ArchiveRequestOption ermittleDavArtAnfrage() {
        if (_isZustandsAnfrage.equals(Art.Zustandsanfrage.name())) {
            return ArchiveRequestOption.NORMAL;
        } else {
            return ArchiveRequestOption.DELTA;
        }
    }

    /**
     * Ermittle DAV-DatenArt fr die Archivanfrage.
     *
     * @return DAV-DatenArt fr die Archivanfrage.
     */
    public ArchiveDataKindCombination ermittleDavDatenArt() {
        List<ArchiveDataKind> liste = new ArrayList<ArchiveDataKind>();

        if (_aktuelleDaten) {
            liste.add(ArchiveDataKind.ONLINE);
        }

        if (_nachgelieferteDaten) {
            liste.add(ArchiveDataKind.ONLINE_DELAYED);
        }

        if (_nachgefordertAktuelleDaten) {
            liste.add(ArchiveDataKind.REQUESTED);
        }

        if (_nachgefordertNachgelieferteDaten) {
            liste.add(ArchiveDataKind.REQUESTED_DELAYED);
        }

        switch (liste.size()) {
        case 1 :
            return new ArchiveDataKindCombination(liste.get(0));

        case 2 :
            return new ArchiveDataKindCombination(liste.get(0), liste.get(1));

        case 3 :
            return new ArchiveDataKindCombination(liste.get(0), liste.get(1), liste.get(2));

        case 4 :
            return new ArchiveDataKindCombination(liste.get(0), liste.get(1), liste.get(2), liste.get(3));

        default :
            return new ArchiveDataKindCombination(ArchiveDataKind.ONLINE);
        }
    }

    /**
     * Ermittle DAV-Prioritt fr die Archivanfrage.
     *
     * @return DAV-Prioritt fr die Archivanfrage.
     */
    public ArchiveQueryPriority ermittleDavPrioritaet() {
        if (_prioritaetDerAnfrage.equals(Prioritaet.Hoch.name())) {
            return ArchiveQueryPriority.HIGH;
        } else if (_prioritaetDerAnfrage.equals(Prioritaet.Mittel.name())) {
            return ArchiveQueryPriority.MEDIUM;
        } else if (_prioritaetDerAnfrage.equals(Prioritaet.Niedrig.name())) {
            return ArchiveQueryPriority.LOW;
        }

        return ArchiveQueryPriority.LOW;
    }

    /**
     * Ermittle DAV-Sortierung fr die Archivanfrage.
     *
     * @return DAV-Sortierung fr die Archivanfrage.
     */
    public ArchiveOrder ermittleDavSortierung() {
        if (_sortierReihenfolge.equals(Sortierung.Datenindex.name())) {
            return ArchiveOrder.BY_INDEX;
        } else {
            return ArchiveOrder.BY_DATA_TIME;
        }
    }

    /**
     * Ermittle DAV-Zeitbereich fr die Archivanfrage.
     *
     * @return DAV-Zeitbereich fr die Archivanfrage.
     */
    public TimingType ermittleDavZeitbereich() {
        if (_zeitBereichDerAnfrage.equals(ZeitBereichDerAnfrage.Datenzeitstempel.name())) {
            return TimingType.DATA_TIME;
        } else if (_zeitBereichDerAnfrage.equals(ZeitBereichDerAnfrage.Archivzeitstempel.name())) {
            return TimingType.ARCHIVE_TIME;
        }

        if (_zeitBereichDerAnfrage.equals(ZeitBereichDerAnfrage.Datenindex.name())) {
            return TimingType.DATA_INDEX;
        }

        return TimingType.DATA_TIME;
    }

    /**
     * Ermittle den Zeitbereich der Archivanfrage.
     *
     * @return Zeitbereich der Archivanfrage als Array [von, bis].
     * </p>
     * -- Bei der relativen Zeitangabe werden berechnete [von Datum, bis Datum] zurck geliefert.
     * </p>
     * -- Bei der absoluten Zeitangabe:
     * </p>
     * -- -- Datenzeitstempel oder Archivzeitstempel:
     * </p>
     * -- -- -- [von Datum, bis Datum]
     * </p>
     * -- -- -- [von Datenstzeanzahl, bis Datum] (falls Datenanzahl begrenzt ist)
     * </p>
     * -- -- Datenindex:
     * </p>
     * -- -- -- [von Index, bis Index]
     * </p>
     * -- -- -- [von Datenstzeanzahl, bis Index] (falls Datenanzahl begrenzt ist)
     */
    public long[] ermittleZeitbereich() {

        // -----------------------------------------
        // Absoluter Zeitbereich:
        if (_istAbsoluteZeitangabe) {

            // --------------------------------------
            // Index:
            if (_zeitBereichDerAnfrage.equals(ZeitBereichDerAnfrage.Datenindex.name())) {

                // von Anzahl der Datenstze bis Index:
                if (_istDatenanzahlBegrenzt) {
                    return new long[] { _anzahlDaten, _bisIndex };
                }

                // von/bis Index:
                else {
                    return new long[] { _vonIndex, _bisIndex };
                }
            }

            // --------------------------------------
            // Datum:
            else {

                // von Anzahl der Datenstze bis Datum:
                if (_istDatenanzahlBegrenzt) {
                    return new long[] { _anzahlDaten, _bisDatum };
                }

                // von/bis Datum:
                else {
                    return new long[] { _vonDatum, _bisDatum };
                }
            }
        }

        // -----------------------------------------
        // Relativer Zeitbereich:
        else {
            return ermittleZeitbereichRelativ();
        }
    }

    /**
     * Ermittle den Zeitbereich der Archivanfrage als Text.
     *
     * @return Zeitbereich der Archivanfrage als Array String[von, bis].
     * Falls von bzw. bis ein Datum darstellt, wird der Wert als Datum dargestellt,
     * sonst - als eine Zahl
     */
    public String[] ermittleZeitbereichAlsText() {
        long[]  zb         = ermittleZeitbereich();
        boolean vonIstZahl = false;
        boolean bisIstZahl = false;

        // Ptfe, wann der Wert eine Zahl darstellt:
        if (_zeitBereichDerAnfrage.equals(ZeitBereichDerAnfrage.Datenindex.name())) {
            vonIstZahl = true;
            bisIstZahl = true;
        } else if (_istDatenanzahlBegrenzt) {
            vonIstZahl = true;
        }

        //
        String von;
        String bis;

        if (vonIstZahl) {
            von = "" + zb[0];
        } else {
            von = DatumFunktionen.getDatumZeitOhneSekundenAlsString(zb[0]);
        }

        if (bisIstZahl) {
            bis = "" + zb[1];
        } else {
            bis = DatumFunktionen.getDatumZeitOhneSekundenAlsString(zb[1]);
        }

        return new String[] { von, bis };
    }

    /**
     * Ermittle relativen Zeitbereich.
     * Relative Zeitbereich ist ber von/bis Monate/Tage/Stunden, die vom aktuellen Zeitpunkt
     * abgezogen werden, definiert.
     *
     * @return Relativer Zeitbereich als Array [von, bis].
     */
    public long[] ermittleZeitbereichRelativ() {
        return ermittleZeitbereichRelativ(_vonMonate, _vonTage, _vonStunden, _istVonAufrunden,  //
                                          _bisMonate, _bisTage, _bisStunden, _istBisAufrunden);
    }

    /**
     * Ermittle relativen Zeitbereich.
     * Relative Zeitbereich ist ber von/bis Monate/Tage/Stunden, die vom aktuellen Zeitpunkt
     * abgezogen werden, definiert.
     *
     * @param vonMonate Siehe {@link #_vonMonate}
     * @param vonTage Siehe {@link #_vonTage}
     * @param vonStunden Siehe {@link #_vonStunden}
     * @param istVonAufrunden Siehe {@link #_istVonAufrunden}
     * @param bisMonate Siehe {@link #_bisMonate}
     * @param bisTage Siehe {@link #_bisTage}
     * @param bisStunden Siehe {@link #_bisStunden}
     * @param istBisAufrunden Siehe {@link #_istBisAufrunden}
     *
     * @return Relativer Zeitbereich als Array [von, bis].
     */
    public static long[] ermittleZeitbereichRelativ(int vonMonate, int vonTage, int vonStunden, boolean istVonAufrunden,  //
                                                    int bisMonate, int bisTage, int bisStunden, boolean istBisAufrunden) {
        long von = ermittleZeitpunktRelativ(vonMonate, vonTage, vonStunden, istVonAufrunden);
        long bis = ermittleZeitpunktRelativ(bisMonate, bisTage, bisStunden, istBisAufrunden);

        return new long[] { von, bis };
    }

    /**
     * Ermittle relativen Zeitpunkt.
     * Relative Zeitpunkt ist ber Monate/Tage/Stunden, die vom aktuellen Zeitpunkt
     * abgezogen werden, definiert.
     *
     * @param monate Vor Monaten.
     * @param tage Vor Tagen.
     * @param stunden Vor Stunden.
     * @param istAufrunden Falls true, wird der aktuelle Zeitpunkt zuerst aufgerundet.
     *
     * @return Relativer Zeitpunkt.
     */
    private static long ermittleZeitpunktRelativ(int monate, int tage, int stunden, boolean istAufrunden) {

        // Aktueller Zeitpunkt:
        // z.B. 10.02.2012 14:39:12:03
        long zeitPunkt = DatumFunktionen.getAktuelleDateTime();

        zeitPunkt = DatumFunktionen.getDatumZeitOhneSekunden(zeitPunkt);

        // -----------------------------------------
        // Aufrunden:
        // -----------------------------------------
        if (istAufrunden) {

            // -------------------------------
            // Aufrundung bis nchstem Monat
            // 10.02.2012 14:39:12:03  -->  01.03.2012  00:00:00:00
            if (monate != 0) {

                // Monatanfang:
                zeitPunkt = DatumFunktionen.getMonatAnfang(zeitPunkt);

                // plus 1 Monat:
                zeitPunkt = DatumFunktionen.getVerschobenenDateTime(zeitPunkt, 1, 0);
            }

            // -------------------------------
            // Aufrundung bis nchstem Tag
            // 10.02.2012 14:39:12:03  -->  11.02.2012  00:00:00:00
            else if (tage != 0) {

                // Zeit 00:
                zeitPunkt = DatumFunktionen.getDatumMitInitZeit(zeitPunkt);

                // plus 1 Tag:
                zeitPunkt = DatumFunktionen.getNaechstenTag(zeitPunkt);
            }

            // -------------------------------
            // Aufrundung bis nchster Stunde
            // 10.02.2012 14:39:12:03  -->  10.02.2012  15:00:00:00
            else if (stunden != 0) {

                // plus 1 Stunde:
                zeitPunkt = zeitPunkt + 1 * 60 * 60 * 1000;

                // Zeit min:sek:millis 00:00:00
                zeitPunkt = DatumFunktionen.setzeZeit(zeitPunkt, DatumFunktionen.getStunden(zeitPunkt), 0);
            }
        }

        // -----------------------------------------
        // Berechnen:
        // -----------------------------------------
        return DatumFunktionen.getVerschobenenRueckwaertsZeitpunkt(zeitPunkt, monate, tage, stunden);
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'anzahlDaten'.
     *
     * @return Wert der Eigenschaft 'anzahlDaten'.
     *
     * @see #_anzahlDaten
     */
    public int getAnzahlDaten() {
        return _anzahlDaten;
    }

    /**
     * Getter fr Eigenschaft 'bisDatum'.
     *
     * @return Wert der Eigenschaft 'bisDatum'.
     *
     * @see #_bisDatum
     */
    public long getBisDatum() {
        return _bisDatum;
    }

    /**
     * Getter fr Eigenschaft 'bisIndex'.
     *
     * @return Wert der Eigenschaft 'bisIndex'.
     *
     * @see #_bisIndex
     */
    public int getBisIndex() {
        return _bisIndex;
    }

    /**
     * Getter fr Eigenschaft 'bisMonate'.
     *
     * @return Wert der Eigenschaft 'bisMonate'.
     *
     * @see #_bisMonate
     */
    public int getBisMonate() {
        return _bisMonate;
    }

    /**
     * Getter fr Eigenschaft 'bisStunden'.
     *
     * @return Wert der Eigenschaft 'bisStunden'.
     *
     * @see #_bisStunden
     */
    public int getBisStunden() {
        return _bisStunden;
    }

    /**
     * Getter fr Eigenschaft 'bisTage'.
     *
     * @return Wert der Eigenschaft 'bisTage'.
     *
     * @see #_bisTage
     */
    public int getBisTage() {
        return _bisTage;
    }

    /**
     * Getter fr Eigenschaft 'isZustandsAnfrage'.
     *
     * @return Wert der Eigenschaft 'isZustandsAnfrage'.
     *
     * @see #_isZustandsAnfrage
     */
    public String getIsZustandsAnfrage() {
        return _isZustandsAnfrage;
    }

    /**
     * Getter fr Eigenschaft 'prioritaetDerAnfrage'.
     *
     * @return Wert der Eigenschaft 'prioritaetDerAnfrage'.
     *
     * @see #_prioritaetDerAnfrage
     */
    public String getPrioritaetDerAnfrage() {
        return _prioritaetDerAnfrage;
    }

    /**
     * Getter fr Eigenschaft 'sortierReihenfolge'.
     *
     * @return Wert der Eigenschaft 'sortierReihenfolge'.
     *
     * @see #_sortierReihenfolge
     */
    public String getSortierReihenfolge() {
        return _sortierReihenfolge;
    }

    /**
     * Getter fr Eigenschaft 'vonDatum'.
     *
     * @return Wert der Eigenschaft 'vonDatum'.
     *
     * @see #_vonDatum
     */
    public long getVonDatum() {
        return _vonDatum;
    }

    /**
     * Getter fr Eigenschaft 'vonIndex'.
     *
     * @return Wert der Eigenschaft 'vonIndex'.
     *
     * @see #_vonIndex
     */
    public int getVonIndex() {
        return _vonIndex;
    }

    /**
     * Getter fr Eigenschaft 'vonMonate'.
     *
     * @return Wert der Eigenschaft 'vonMonate'.
     *
     * @see #_vonMonate
     */
    public int getVonMonate() {
        return _vonMonate;
    }

    /**
     * Getter fr Eigenschaft 'vonStunden'.
     *
     * @return Wert der Eigenschaft 'vonStunden'.
     *
     * @see #_vonStunden
     */
    public int getVonStunden() {
        return _vonStunden;
    }

    /**
     * Getter fr Eigenschaft 'vonTage'.
     *
     * @return Wert der Eigenschaft 'vonTage'.
     *
     * @see #_vonTage
     */
    public int getVonTage() {
        return _vonTage;
    }

    /**
     * Getter fr Eigenschaft 'zeitBereichDerAnfrage'.
     *
     * @return Wert der Eigenschaft 'zeitBereichDerAnfrage'.
     *
     * @see #_zeitBereichDerAnfrage
     */
    public String getZeitBereichDerAnfrage() {
        return _zeitBereichDerAnfrage;
    }

    /**
     * Getter fr Eigenschaft 'aktuelleDaten'.
     *
     * @return Wert der Eigenschaft 'aktuelleDaten'.
     *
     * @see #_aktuelleDaten
     */
    public boolean isAktuelleDaten() {
        return _aktuelleDaten;
    }

    /**
     * Getter fr Eigenschaft 'istAbsoluteZeitangabe'.
     *
     * @return Wert der Eigenschaft 'istAbsoluteZeitangabe'.
     *
     * @see #_istAbsoluteZeitangabe
     */
    public boolean isIstAbsoluteZeitangabe() {
        return _istAbsoluteZeitangabe;
    }

    /**
     * Getter fr Eigenschaft 'istBisAufrunden'.
     *
     * @return Wert der Eigenschaft 'istBisAufrunden'.
     *
     * @see #_istBisAufrunden
     */
    public boolean isIstBisAufrunden() {
        return _istBisAufrunden;
    }

    /**
     * Getter fr Eigenschaft 'istDatenanzahlBegrenzt'.
     *
     * @return Wert der Eigenschaft 'istDatenanzahlBegrenzt'.
     *
     * @see #_istDatenanzahlBegrenzt
     */
    public boolean isIstDatenanzahlBegrenzt() {
        return _istDatenanzahlBegrenzt;
    }

    /**
     * Getter fr Eigenschaft 'istVonAufrunden'.
     *
     * @return Wert der Eigenschaft 'istVonAufrunden'.
     *
     * @see #_istVonAufrunden
     */
    public boolean isIstVonAufrunden() {
        return _istVonAufrunden;
    }

    /**
     * Getter fr Eigenschaft 'nachgefordertAktuelleDaten'.
     *
     * @return Wert der Eigenschaft 'nachgefordertAktuelleDaten'.
     *
     * @see #_nachgefordertAktuelleDaten
     */
    public boolean isNachgefordertAktuelleDaten() {
        return _nachgefordertAktuelleDaten;
    }

    /**
     * Getter fr Eigenschaft 'nachgefordertNachgelieferteDaten'.
     *
     * @return Wert der Eigenschaft 'nachgefordertNachgelieferteDaten'.
     *
     * @see #_nachgefordertNachgelieferteDaten
     */
    public boolean isNachgefordertNachgelieferteDaten() {
        return _nachgefordertNachgelieferteDaten;
    }

    /**
     * Getter fr Eigenschaft 'nachgelieferteDaten'.
     *
     * @return Wert der Eigenschaft 'nachgelieferteDaten'.
     *
     * @see #_nachgelieferteDaten
     */
    public boolean isNachgelieferteDaten() {
        return _nachgelieferteDaten;
    }

    //~ METHODEN ==============================================================

    /**
     * Ermittle, ob der Zeitbereich des Archivs korrekt ist.
     *
     * @return true, falls der Zeitbereich des Archivs korrekt ist,
     * false - sonst.
     */
    public boolean istZeitbereichKorrekt() {

        // Falls die Anzahl der Datenstze begrenzt ist:
        if (_istDatenanzahlBegrenzt) {
            return true;
        }

        // sonst:
        else {
            long[] zb = ermittleZeitbereich();

            return zb[0] < zb[1];
        }
    }

    //~ SET METHODEN ==========================================================

    /**
     * Setter fr Eigenschaft 'aktuelleDaten'.
     *
     * @param aktuelleDaten Wert zum Setzen der Eigenschaft 'aktuelleDaten'.
     *
     * @see #_aktuelleDaten
     */
    public void setAktuelleDaten(boolean aktuelleDaten) {
        _aktuelleDaten = aktuelleDaten;
    }

    /**
     * Setter fr Eigenschaft 'anzahlDaten'.
     *
     * @param anzahlDaten Wert zum Setzen der Eigenschaft 'anzahlDaten'.
     *
     * @see #_anzahlDaten
     */
    public void setAnzahlDaten(int anzahlDaten) {
        _anzahlDaten = anzahlDaten;
    }

    /**
     * Setter fr Eigenschaft 'bisDatum'.
     *
     * @param bisDatum Wert zum Setzen der Eigenschaft 'bisDatum'.
     *
     * @see #_bisDatum
     */
    public void setBisDatum(long bisDatum) {
        _bisDatum = bisDatum;
    }

    /**
     * Setter fr Eigenschaft 'bisIndex'.
     *
     * @param bisIndex Wert zum Setzen der Eigenschaft 'bisIndex'.
     *
     * @see #_bisIndex
     */
    public void setBisIndex(int bisIndex) {
        _bisIndex = bisIndex;
    }

    /**
     * Setter fr Eigenschaft 'bisMonate'.
     *
     * @param bisMonate Wert zum Setzen der Eigenschaft 'bisMonate'.
     *
     * @see #_bisMonate
     */
    public void setBisMonate(int bisMonate) {
        _bisMonate = bisMonate;
    }

    /**
     * Setter fr Eigenschaft 'bisStunden'.
     *
     * @param bisStunden Wert zum Setzen der Eigenschaft 'bisStunden'.
     *
     * @see #_bisStunden
     */
    public void setBisStunden(int bisStunden) {
        _bisStunden = bisStunden;
    }

    /**
     * Setter fr Eigenschaft 'bisTage'.
     *
     * @param bisTage Wert zum Setzen der Eigenschaft 'bisTage'.
     *
     * @see #_bisTage
     */
    public void setBisTage(int bisTage) {
        _bisTage = bisTage;
    }

    /**
     * Setter fr Eigenschaft 'istAbsoluteZeitangabe'.
     *
     * @param istAbsoluteZeitangabe Wert zum Setzen der Eigenschaft 'istAbsoluteZeitangabe'.
     *
     * @see #_istAbsoluteZeitangabe
     */
    public void setIstAbsoluteZeitangabe(boolean istAbsoluteZeitangabe) {
        _istAbsoluteZeitangabe = istAbsoluteZeitangabe;
    }

    /**
     * Setter fr Eigenschaft 'istBisAufrunden'.
     *
     * @param istBisAufrunden Wert zum Setzen der Eigenschaft 'istBisAufrunden'.
     *
     * @see #_istBisAufrunden
     */
    public void setIstBisAufrunden(boolean istBisAufrunden) {
        _istBisAufrunden = istBisAufrunden;
    }

    /**
     * Setter fr Eigenschaft 'istDatenanzahlBegrenzt'.
     *
     * @param istDatenanzahlBegrenzt Wert zum Setzen der Eigenschaft 'istDatenanzahlBegrenzt'.
     *
     * @see #_istDatenanzahlBegrenzt
     */
    public void setIstDatenanzahlBegrenzt(boolean istDatenanzahlBegrenzt) {
        _istDatenanzahlBegrenzt = istDatenanzahlBegrenzt;
    }

    /**
     * Setter fr Eigenschaft 'istVonAufrunden'.
     *
     * @param istVonAufrunden Wert zum Setzen der Eigenschaft 'istVonAufrunden'.
     *
     * @see #_istVonAufrunden
     */
    public void setIstVonAufrunden(boolean istVonAufrunden) {
        _istVonAufrunden = istVonAufrunden;
    }

    /**
     * Setter fr Eigenschaft 'isZustandsAnfrage'.
     *
     * @param isZustandsAnfrage Wert zum Setzen der Eigenschaft 'isZustandsAnfrage'.
     *
     * @see #_isZustandsAnfrage
     */
    public void setIsZustandsAnfrage(String isZustandsAnfrage) {
        _isZustandsAnfrage = isZustandsAnfrage;
    }

    /**
     * Setter fr Eigenschaft 'nachgefordertAktuelleDaten'.
     *
     * @param nachgefordertAktuelleDaten Wert zum Setzen der Eigenschaft 'nachgefordertAktuelleDaten'.
     *
     * @see #_nachgefordertAktuelleDaten
     */
    public void setNachgefordertAktuelleDaten(boolean nachgefordertAktuelleDaten) {
        _nachgefordertAktuelleDaten = nachgefordertAktuelleDaten;
    }

    /**
     * Setter fr Eigenschaft 'nachgefordertNachgelieferteDaten'.
     *
     * @param nachgefordertNachgelieferteDaten Wert zum Setzen der Eigenschaft 'nachgefordertNachgelieferteDaten'.
     *
     * @see #_nachgefordertNachgelieferteDaten
     */
    public void setNachgefordertNachgelieferteDaten(boolean nachgefordertNachgelieferteDaten) {
        _nachgefordertNachgelieferteDaten = nachgefordertNachgelieferteDaten;
    }

    /**
     * Setter fr Eigenschaft 'nachgelieferteDaten'.
     *
     * @param nachgelieferteDaten Wert zum Setzen der Eigenschaft 'nachgelieferteDaten'.
     *
     * @see #_nachgelieferteDaten
     */
    public void setNachgelieferteDaten(boolean nachgelieferteDaten) {
        _nachgelieferteDaten = nachgelieferteDaten;
    }

    /**
     * Setter fr Eigenschaft 'prioritaetDerAnfrage'.
     *
     * @param prioritaetDerAnfrage Wert zum Setzen der Eigenschaft 'prioritaetDerAnfrage'.
     *
     * @see #_prioritaetDerAnfrage
     */
    public void setPrioritaetDerAnfrage(String prioritaetDerAnfrage) {
        _prioritaetDerAnfrage = prioritaetDerAnfrage;
    }

    /**
     * Setter fr Eigenschaft 'sortierReihenfolge'.
     *
     * @param sortierReihenfolge Wert zum Setzen der Eigenschaft 'sortierReihenfolge'.
     *
     * @see #_sortierReihenfolge
     */
    public void setSortierReihenfolge(String sortierReihenfolge) {
        _sortierReihenfolge = sortierReihenfolge;
    }

    /**
     * Setter fr Eigenschaft 'vonDatum'.
     *
     * @param vonDatum Wert zum Setzen der Eigenschaft 'vonDatum'.
     *
     * @see #_vonDatum
     */
    public void setVonDatum(long vonDatum) {
        _vonDatum = vonDatum;
    }

    /**
     * Setter fr Eigenschaft 'vonIndex'.
     *
     * @param vonIndex Wert zum Setzen der Eigenschaft 'vonIndex'.
     *
     * @see #_vonIndex
     */
    public void setVonIndex(int vonIndex) {
        _vonIndex = vonIndex;
    }

    /**
     * Setter fr Eigenschaft 'vonMonate'.
     *
     * @param vonMonate Wert zum Setzen der Eigenschaft 'vonMonate'.
     *
     * @see #_vonMonate
     */
    public void setVonMonate(int vonMonate) {
        _vonMonate = vonMonate;
    }

    /**
     * Setter fr Eigenschaft 'vonStunden'.
     *
     * @param vonStunden Wert zum Setzen der Eigenschaft 'vonStunden'.
     *
     * @see #_vonStunden
     */
    public void setVonStunden(int vonStunden) {
        _vonStunden = vonStunden;
    }

    /**
     * Setter fr Eigenschaft 'vonTage'.
     *
     * @param vonTage Wert zum Setzen der Eigenschaft 'vonTage'.
     *
     * @see #_vonTage
     */
    public void setVonTage(int vonTage) {
        _vonTage = vonTage;
    }

    /**
     * Setter fr Eigenschaft 'zeitBereichDerAnfrage'.
     *
     * @param zeitBereichDerAnfrage Wert zum Setzen der Eigenschaft 'zeitBereichDerAnfrage'.
     *
     * @see #_zeitBereichDerAnfrage
     */
    public void setZeitBereichDerAnfrage(String zeitBereichDerAnfrage) {
        _zeitBereichDerAnfrage = zeitBereichDerAnfrage;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 27.02.12
