/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.inovat.lib.debug;

//~ NICHT JDK IMPORTE =========================================================

import org.eclipse.core.runtime.MultiStatus;

//~ KLASSEN ===================================================================

/**
 * Klasse erzeugt und verwaltet eine MultiMeldung.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 12 $ / $Date: 2013-09-24 13:23:48 +0200 (Di, 24. Sep 2013) $ / ($Author: LG $)
 */
public class MultiMeldung extends MultiStatus {

    /** Zeit der letzten Meldungsnderung. */
    private long _meldungsZeit;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse MultiMeldung.
     *
     * @param pluginID ID vom PlugIn.
     * @param text berschrift der MultiMeldung.
     */
    public MultiMeldung(String pluginID, String text) {
        super(pluginID, 0, text, null);
        aendereMeldungsZeit();
    }

    /**
     * Erzeugt eine Instanz der Klasse MultiMeldung.
     * Fr diese MultiMeldung wird ein Kind mit dem gleichen Texterzeugt.
     *
     * @param pluginID ID vom PlugIn.
     * @param typ Typ vom Kind. Kann eine der Konstanten Log.OK, Log.INFO, Log.WARNUNG, Log.FEHLER, Log.ABBRUCH sein.
     * @param text berschrift der MultiMeldung und des Kindes.
     */
    public MultiMeldung(String pluginID, int typ, String text) {
        this(pluginID, text);
        this.add(Log.erzeugeMeldung(typ, this.getPlugin(), text));
    }

    /**
     * Erzeugt eine Instanz der Klasse MultiMeldung.
     * Fr diese MultiMeldung wird ein Kind mit dem gleichen Text und eine Exception erzeugt.
     *
     * @param pluginID ID vom PlugIn.
     * @param typ Typ vom Kind. Kann eine der Konstanten Log.OK, Log.INFO, Log.WARNUNG, Log.FEHLER, Log.ABBRUCH sein.
     * @param text berschrift der MultiMeldung und des Kindes.
     * @param e Exception.
     */
    public MultiMeldung(String pluginID, int typ, String text, Exception e) {
        this(pluginID, text);
        this.add(Log.erzeugeMeldung(typ, this.getPlugin(), text, e));
    }

    //~ METHODEN ==============================================================

    /**
     * In der Meldungszeit wird die aktuelle Zeit gespeichert.
     */
    private void aendereMeldungsZeit() {
        _meldungsZeit = System.currentTimeMillis();
    }

    /**
     * Wird eine _Fehler-Meldung zur MultiMeldung hinzugefgt.
     *
     * @param text Meldungstext.
     */
    public void fehler(String text) {
        this.add(Log.erzeugeMeldung(Log.FEHLER, this.getPlugin(), text));
        aendereMeldungsZeit();
    }

    /**
     * Wird eine _Fehler-Meldung mit Exception zur MultiMeldung hinzugefgt.
     *
     * @param text Meldungstext.
     * @param e  Exception.
     */
    public void fehler(String text, Exception e) {
        this.add(Log.erzeugeMeldung(Log.FEHLER, this.getPlugin(), text, e));
        aendereMeldungsZeit();
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'meldungsZeit'.
     *
     * @return Wert der Eigenschaft 'meldungsZeit'.
     *
     * @see #_meldungsZeit
     */
    public long getMeldungsZeit() {
        return _meldungsZeit;
    }

    //~ METHODEN ==============================================================

    /**
     * Fge eine MultiMeldung zu dieser MultiMeldung zu.
     *
     * @param multiMeldung MultiMeldung, die hinzugefgt wird.
     */
    public void hinzufuegenMultiMeldung(MultiMeldung multiMeldung) {
        this.add(multiMeldung);
        aendereMeldungsZeit();
    }

    /**
     * Wird eine Info-Meldung zur MultiMeldung hinzugefgt.
     *
     * @param text Meldungstext.
     */
    public void info(String text) {
        this.add(Log.erzeugeMeldung(Log.INFO, this.getPlugin(), text));
        aendereMeldungsZeit();
    }

    /**
     * Wird eine Info-Meldung mit Exception zur MultiMeldung hinzugefgt.
     *
     * @param text Meldungstext.
     * @param e  Exception.
     */
    public void info(String text, Exception e) {
        this.add(Log.erzeugeMeldung(Log.INFO, this.getPlugin(), text, e));
        aendereMeldungsZeit();
    }

    /**
     * Wird eine Warnung-Meldung zur MultiMeldung hinzugefgt.
     *
     * @param text Meldungstext.
     */
    public void warnung(String text) {
        this.add(Log.erzeugeMeldung(Log.WARNUNG, this.getPlugin(), text));
        aendereMeldungsZeit();
    }

    /**
     * Wird eine Warnung-Meldung mit Exception zur MultiMeldung hinzugefgt.
     *
     * @param text Meldungstext.
     * @param e  Exception.
     */
    public void warnung(String text, Exception e) {
        this.add(Log.erzeugeMeldung(Log.WARNUNG, this.getPlugin(), text, e));
        aendereMeldungsZeit();
    }

    /**
     * Zeige die MultiMeldung im ErrorLog.
     * Falls die MultiMeldung nur ein Kind hat und
     * die berschriften gleich sind,
     * wird nur dieses Kind angezeigt.
     */
    public void zeige() {
        if ((this.getChildren().length == 1) && (this.getChildren()[0].getMessage().equals(this.getMessage()))) {
            Log.zeige(this.getChildren()[0]);
        } else {
            Log.zeige(this);
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 04.11.09
