/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.bast.gui;

//~ JDK IMPORTE ===============================================================

import java.util.List;

//~ INTERFACES ================================================================

/**
 * Interface definiert die Export-Methoden.
 *
 * Um das ExportGUI fr einen bestimmten Export zu benutzen,
 * definiert man in der Datei pugin.xml ein Extension 'View'.
 * In der zugehrigen Klasse in der Methode createPartControl()
 * erzeugt man eine ExportGUI-Instanz. In den Konstruktor wird dabei
 * eine Instanz der Klasse bergeben, die dieses Interface implementiert.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 88 $ / $Date: 2012-03-09 16:20:38 +0100 (Fr, 09 Mrz 2012) $ / ($Author: LG $)
 */
public interface IExportEinstellungen {

    /**
     * Ermittelt die Liste alle Exportausgabedateien.
     *
     * Relevant nur, wenn die Methode istErstellungMehrereDateienMoeglich()() true zurck gibt,
     *
     * @see #istErstellungMehrereDateienMoeglich()()
     *
     * @return Liste alle Exportausgabedateien.
     */
    public List<String> ermittleAlleAusgabeDateien();

    /**
     * Ermittle den Datei-Typ. Entsprechend diesem Typ wird im Windows
     * das Standard-Programm fr die weitere Bearbeitung der Datei aufgemacht.
     *
     * Relevant nur, wenn die Methode istBearbeitenErlaubt() true zurck gibt.
     * @see #istBearbeitenErlaubt()
     *
     * @return Typ der Datei, um im Windows das entsprechende Standard-Programm fr die Datei zu ffnen.
     */
    public String ermittleDateiTyp();

    /**
     * Ermittle den Default-Dateinamen, wohin die Messstellene exportiert wurden.
     *
     * @return Default-Dateinamen, wohin die Messstellene exportiert werden.
     */
    public String ermittleDefaultExportDatei();

    /**
     * Ermittle Dateitypen fr den Dateiauswahldialog.
     *
     * @return Dateitypen fr den Dateiauswahldialog.
     */
    public String[] ermittleFilterExtensions();

    /**
     * Ermittle ID des Parameters, ber das die String-Representation
     * des Parameters gespeichert wird.
     * Der Parameter soll als Konstante in der Klasse GrundEinstellungen
     * definiert werden.
     *
     * @return ID des Parameters, ber das die String-Representation
     * des Parameters gespeichert wird.
     */
    public String ermittleParameterID();

    /**
     * Ermittle den Text fr das GUI-Label Z'eitbereich'.
     * Man kann mehrere Zeile angeben indem man
     * de.inovat.buv.plugin.lib.konstanten.Konstanten.NEWLINE
     * benutzt.
     *
     * Relevant nur, wenn die Methode istZeitbereichsAngabeMoeglich() true zurck gibt.
     * @see #istZeitbereichsAngabeMoeglich()
     *
     * @return Text fr das Label 'Zeitbereich'.
     */
    public String ermittleTextZeitbereich();

    /**
     * Ermittle Zeitangabe-Ende.
     *
     * Relevant nur, wenn die Methode istZeitbereichsAngabeMoeglich() true zurck gibt.
     * @see #istZeitbereichsAngabeMoeglich()
     *
     * @return Zeitangabe-Ende.
     */
    public long ermittleZeitangabeBis();

    /**
     * Ermittle Zeitangabe-Beginn.
     *
     * Relevant nur, wenn die Methode istZeitbereichsAngabeMoeglich() true zurck gibt.
     * @see #istZeitbereichsAngabeMoeglich()
     *
     * @return Zeitangabe-Beginn.
     */
    public long ermittleZeitangabeVon();

    /**
     * Legt fest, ob die Bearbeiten-Funktion fr die Export-Datei erlaubt ist.
     * Relevant nur, wenn die Methode ermittleAlleAusgabeDateien() sowie
     *                  istErstellungMehrereDateienMoeglich()() true zurck gibt!
     * @return true, falls die Bearbeiten-Funktion fr die Export-Datei erlaubt ist,
     * false - sonst.
     */
    public boolean istBearbeitenErlaubt();

    /**
     * Legt fest, ob Dateiauswahl oder Ordnerauswahl mglich ist.
     * @return true, falls Dateiauswahl,
     * false - sonst.
     */
    public boolean istDateiAuswahl();

    /**
     * Legt fest, ob die Druck-Funktion fr die Export-Datei erlaubt ist.
     *
     * @return true, falls die Druck-Funktion fr die Export-Datei erlaubt ist,
     * false - sonst.
     */
    public boolean istDruckenErlaubt();

    /**
     * Legt fest, ob die EMail-Funktion fr die Export-Datei erlaubt ist.
     *
     * @return true, falls die EMail-Funktion fr die Export-Datei erlaubt ist,
     * false - sonst.
     */
    public boolean istEmailErlaubt();

    /**
     * Legt fest, ob die Export-Ausgabe in mehrere Dateien mglich ist.
     *
     * @return true, falls die Export-Ausgabe in mehrere Dateien mglich ist,
     * false sonst.
     */
    public boolean istErstellungMehrereDateienMoeglich();

    /**
     * Legt fest, ob die Gruppe 'Zeitbereich' im GUI angezeigt wird.
     * Falls ja, muss man noch sinnvoll die Methoden:
     * ermittleTextZeitbereich(), ermittleZeitangabeVon()
     * und ermittleZeitangabeBis() implementieren.
     *
     * @return true, falls die Gruppe 'Zeitbereich' im GUI angezeigt wird,
     * false - sonst.
     */
    public boolean istZeitbereichsAngabeMoeglich();
}


//~Formatiert mit 'inovat Kodierkonvention' am 21.10.13
